#ifndef EXONMAP_H
#define EXONMAP_H

#include <exception>


// Exon map class.
class ExonMap {

	public:
		ExonMap(unsigned __chromlength) : chromlength(__chromlength){ std::cout<<"Constructor called!"<<std::endl; }
		~ExonMap(){ std::cout<<"Destructor called!"<<std::endl; }
		std::vector<std::pair<double, double> > positions;
		std::vector<std::pair<double, double> > exondensities;
		unsigned chromlength;
		void ReadIntervals(const char * filename);
		void ReadDensities(const char * filename);
		typedef bool result_type;
		inline bool operator()(double pos) const {
			auto exonitr=std::upper_bound(positions.begin(), positions.end(), pos, [](const double val, const std::pair<double, double> & interval){ return val < interval.second; } );
			return (exonitr!=positions.end() && exonitr->first <= pos) ? true : false;
			}
		double getDensity(double pos){
			auto exonitr=std::upper_bound(exondensities.begin(), exondensities.end(), pos, [](const double val, const std::pair<double, double> & interval){ return val < interval.first; } );
			return ( exonitr==exondensities.end() ) ? exondensities.rbegin()->second : exonitr->second;
			}
	};

void ExonMap::ReadIntervals(const char * filename)
{
  std::ifstream input;
  input.open(filename);
  if ( !input.is_open() )
  {
    throw std::runtime_error("Exon interval file " + std::string(filename) + " could not be opened!");
  }
	positions.clear();
	std::string line;
	unsigned prevend=0;
	while ( getline(input, line) ){
		std::istringstream ss(line);
		if ( ss.good() ){
			std::string scaff;
			int start=0, end=0;
			ss>>scaff>>start>>end;
			if (start<prevend){
            throw std::runtime_error("Exon interval file " + std::string(filename) + " is not sorted!");
				}
			prevend=end;
			int length=end-start;
			if (length<1){
				std::cout<<"Invalid range "<<scaff<<":"<<start<<"-"<<end<<" in exon bed file!\n";
				continue;
				}
			positions.push_back( std::pair<double, double>( start/(double)chromlength, end/(double)chromlength ) );
			}
		}
	if ( input.bad() ){
      throw std::runtime_error("Exon interval file " + std::string(filename) + " has formatting issues!");
		}
	input.close();
	std::cout<<"Exon interval file "<<filename<<" successfully processed and "<<positions.size()<<" exons read in."<<std::endl;
	return;
	};

void ExonMap::ReadDensities(const char * filename){
	std::ifstream input;
	input.open(filename);
	if ( !input.is_open() ){
      throw std::runtime_error("Exon density file " + std::string(filename) + " could not be opened!");
		}
	positions.clear();
	std::string line;
	unsigned prevend=0;
	while ( getline(input, line) ){
		std::istringstream ss(line);
		if ( ss.good() ){
			std::string scaff;
			int start=0, end=0;
			double density;
			ss>>scaff>>start>>end>>density;
			if (start!=prevend){
            throw std::runtime_error("Exon density file " + std::string(filename) + " is not contiguous!");
				}
			prevend=end;
			int length=end-start;
			if (length<1){
				std::cout<<"Invalid range "<<scaff<<":"<<start<<"-"<<end<<" in exon bed file!\n";
				continue;
				}
			exondensities.push_back( std::pair<double, double>( end/(double)chromlength, density ) );
			}
		}
	if (prevend!=chromlength){
      throw std::runtime_error("Exon density file " + std::string(filename) + " is not of same length as the chromosome!");
		}	
	if ( input.bad() ){
      throw std::runtime_error("Exon density file " + std::string(filename) + " has formatting issues!");
		}
	input.close();
	std::cout<<"Exon density file "<<filename<<" successfully processed and "<<exondensities.size()<<" windows read in."<<std::endl;
	return;
	};

#endif

