#ifndef RECMAP_H
#define RECMAP_H

#include <exception>


// recombination map class.
class RecombinationMap {

	public:
		RecombinationMap(gsl_rng * __r) : r(__r){}
		~RecombinationMap(){}
		gsl_rng * const r;
		gsl_ran_discrete_t * preproc=NULL;
		std::vector<std::pair<double, double> > positions;
		std::vector<double> weights;
		unsigned categories=0, chromlength=0;
		double sumrecrate=0, meanrecrate=0;
		void ReadfromFile(const char * filename);
		const double getlittler(){ return sumrecrate; }
		const unsigned getchromlength(){ return chromlength; }
		void freepreproc() { if (preproc!=NULL) gsl_ran_discrete_free(preproc); }
		typedef double result_type;
		inline double operator()() const {
			size_t cat=gsl_ran_discrete (r, preproc);
			return ( gsl_ran_flat(r, positions[cat].first, positions[cat].second) );
			}
	};

void RecombinationMap::ReadfromFile(const char * filename){
	std::ifstream input;
	input.open(filename);
	if ( !input.is_open() ){
     throw std::runtime_error("Recombination rate file " + std::string(filename) + " could not be opened!");
		}
	std::string line;
	while ( getline(input, line) ){
		std::istringstream ss(line);
		if ( ss.good() ){
			std::string scaff;
			int start=0, end=0;
			double recrate;
			ss>>scaff>>start>>end>>recrate;
			int length=end-start;
			if (length<1){
				std::cout<<"Invalid range "<<scaff<<":"<<start<<"-"<<end<<" in recombination rate bed file!\n";
				continue;
				}
			weights.push_back(recrate * length);
			positions.push_back( std::pair<double, double>( (double)chromlength, (double)(chromlength+length) ) );
			sumrecrate+=recrate * length;
			chromlength+=length;
			++categories;
			}
		}
	if ( input.bad() ){
     throw std::runtime_error("Recombination rate file " + std::string(filename) + " has formatting issues!");
		}
	input.close();
	double * weightarray=new double [ weights.size() ]();
	short unsigned int index=0;
	for (auto it=weights.begin(); it!=weights.end(); ++it){ weightarray[index++]=*it; }
	preproc=gsl_ran_discrete_preproc(weights.size(), weightarray);
	delete [] weightarray;
	for (auto it=positions.begin(); it!=positions.end(); ++it){
		it->first/=(double)chromlength;
		it->second/=(double)chromlength;
		}
	std::cout<<chromlength<<", "<<sumrecrate<<std::endl;
    meanrecrate=sumrecrate/(double)chromlength;
	std::cout<<"Recombination rate file "<<filename<<" successfully processed!"<<std::endl;
	return;
	};

#endif

