/*
 Code used to calculate B in Rettelbach et al, How linked selection shapes the diversity landscape in Ficedula flycatchers
 calculates analytical bgs for given recombination/selection target map and sweeps at substitutions - different alpha for different categories and different split times possible
 Uses equations from Elyashiv et al. (2016)
 Fly catcher parameters are hard coded. Needs to be changed for other organism.
 
 */

#include <numeric>
#include <functional>
#include <cassert>
#include <cmath>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <utility>
#include <time.h>
#include <exception>
#include <stdlib.h> 
#include <vector>
#include <string>
#include <exception>
#include <stdexcept> 
using namespace std; 



int main(int argc, char ** argv) try
{
  if (argc != 11)
  {
    std::cerr << "Usage: exonmap_filename recmap_filename substitutions_filename s_pos alpha1 alpha2 nr_of_windows_cs nr_of_windows_re outfile\n";
 
  }
  int argument=1;
  const char * csmapfile=argv[argument++]; //file with nr of con sites within one window, one number per line!
  const char * recmapfile=argv[argument++]; // file with rec rate, one number per line, length of window in cM!
  const char * subsfile_ns=argv[argument++]; // file with substitutions set 1
  const char * subsfile_up=argv[argument++]; // file with substitutions set 2
  const double s_pos=atof(argv[argument++]); // positive sel strength
  const double alpha = atof(argv[argument++]); //fraction of beneficial substitutions
  const double alpha2 = atof(argv[argument++]);
  const int windnr=atof(argv[argument++]); // nr of windows -has to fit to the cs input file!!
  const int windnr_rec=atof(argv[argument++]); // nr of windows -has to fit to the rec input file!!
  const char * outfilepre = argv[argument++];
  
  const int Ne= 450000; 
  const double splittime= 320000;
  const double splittime2= 2500000;
  const double mu=0.0000000046;
 
      std::string outfile(outfilepre);
    outfile = outfile + ".txt";
    std::string outfile2(outfilepre);
    outfile2 = outfile2 + "_1to200.txt";
  
  // ++++++++++++ reading stuff from input files ++++++++++++++++++++++

  //read recrates from file  
  vector<double> recrates1 (windnr_rec); //vector containing recrates
  int recindex=0;
  string onevalue;
	  ifstream infile;
	  infile.open (recmapfile);
	  if ( !infile.is_open() ){
	    throw std::runtime_error("recombination file not found");
	    }
	  while(getline(infile,onevalue)) // get all the lines.
	  {
	  
		  recrates1[recindex]=atof(onevalue.c_str());
		  recindex++;
		
	  }
	  infile.close();

//read cs from file
  vector<double> consites (windnr); //vector containing nr of conserved sites 
  int conindex=0;
  string onevalue2;
	  ifstream infile2;
	  infile2.open (csmapfile);
	  if ( !infile2.is_open() ){
	    throw std::runtime_error("selection targets file not found");
	    }
	  while(getline(infile2,onevalue2)) // To get you all the lines.
	  {
		
		  consites[conindex]=atof(onevalue2.c_str());
		  conindex++;
		  
	  }
	  infile2.close();
	  
	
// convert rec windows to cs windows
  vector<double> recrates (windnr); // new vector with same size as cs 
  if (windnr>windnr_rec){  
      
    
      int wratio= windnr/windnr_rec; //how many cs windows in one rec window?
      int indcon; //indexconverter
      for (int p1=0; p1<windnr_rec; p1++){ //over all rec windows
	for (int p2=0; p2<wratio; p2++) { //new nr of windows for each old one
	  indcon= p1*wratio+p2;
	  recrates[indcon]=recrates1[p1];
	  }      
      }
  } else recrates=recrates1;
  
  //read subs from file  
  vector<double> subs (windnr); //vector containing subs
  int subindex=0;
  string onevalue3;
	  ifstream infile3;
	  infile3.open (subsfile_ns);
	  if ( !infile3.is_open() ){
	    throw std::runtime_error("substitutions file not found");
	    }
	  while(getline(infile3,onevalue3)) // get all the lines.
	  {
	  
		  subs[subindex]=atof(onevalue3.c_str());
		  subindex++;
		
	  }
	  infile3.close();
	  
	    //read subs from file  
  vector<double> subs2 (windnr); //vector containing subs
  int subindex2=0;
  string onevalue4;
	  ifstream infile4;
	  infile4.open (subsfile_up);
	  if ( !infile4.is_open() ){
	    throw std::runtime_error("substitutions file not found");
	    }
	  while(getline(infile4,onevalue4)) // get all the lines.
	  {
	  
		  subs2[subindex2]=atof(onevalue4.c_str());
		  subindex2++;
		
	  }
	  infile4.close();
	  

//++++++++++++ recombination distances ++++++++++++++++++++

//calculate all centimorgans between all windows
  vector< vector<double> > recmatrix(windnr, vector<double>(windnr));

// sum up all the rec rates of windows between two sites, this takes some time
 for (int i=0; i<windnr ; i++){
    
   //for (int j=max(0,i-2000); j<min(windnr,i+2000) ; j++){
   for (int j=0; j<windnr ; j++){
     
     recmatrix[i][j]= 0.25*(recrates[i]); //mean distance within own window
     if (i>j) {
       recmatrix[i][j]=recmatrix[i][j]+ 0.5*(recrates[j]); //+ mean rdistance final window
       for (int k=i-1; k>j; k--){
	 recmatrix[i][j]=recmatrix[i][j]+recrates[k]; //add all windows between i and j
       }
     }
     if (i<j) {
       recmatrix[i][j]=recmatrix[i][j]+ 0.5*(recrates[j]); //+ mean distance final window
       for (int k=i+1; k<j; k++){
	 recmatrix[i][j]=recmatrix[i][j]+recrates[k]; //add all windows between i and j
       }    
     }
   }
    
    if (i % 100 == 0 ) {cout << "window " << i << "\n";}
 }

 
 // +++++++++++ calculate B and S for all the windows +++++++++++++++++
 
 double B;
 double frac; // store the fraction here
 double frac1; //these are for the different mutation effects
 double frac2; 
 double frac3; 
 double frac4; 
 double Bexp; //the exponent for calculating B
 double recprob; //rec prob between two sites
 double S; // two different S possible
 double S2;
 double B_tot;
 double tau= 2*(log(4*Ne*s_pos)+0.577-(1/(4*Ne*s_pos)))/s_pos; // after Elyasiv 16
 double Ssum; //for calculating S
 double Ssum2; //for calculating S2
 double prod; //store intermediate calc result
 double prod2; //store intermediate calc result 
 vector<double> pivector (windnr);
 double s1=1.0/Ne; //these are the four categories of deleterious mutations. 
 double s2=10.0/Ne;
 double s3=100.0/Ne;
 double s4=500.0/Ne; 
 
 
   std::ofstream myfile( outfile.c_str() );
  //myfile.open (outfile);
   myfile << "s_pos=" << s_pos << " alpha1=" << alpha << " alpha2=" << alpha2 << "\n";
   
 for (int i2=0; i2<windnr; i2++){
  Bexp=0.0; //set to 0
  Ssum=0.0; //set to 0
  Ssum2=0.0;
  B=0.0;
  S=0.0;
  S2=0.0;
  B_tot=0.0;
    
    for (int j2=0; j2<windnr; j2++){ //sum over all columns j2 in row i2
    recprob= (1-exp(-2*recmatrix[i2][j2]/100))/2;
    
    
    //BGS
    //this is the influence of one mut in window j2 on a site in window i2, times the nr of sel. sites in window j2
    //calculate for different categories of mutations and weight them according to dfe
    frac1=(mu/(s1*pow(1+(1-s1)*recprob/s1,2))) * consites[j2];//version from McVicker/Nordborg 
    frac2=(mu/(s2*pow(1+(1-s2)*recprob/s2,2))) * consites[j2];
    frac3=(mu/(s3*pow(1+(1-s3)*recprob/s3,2))) * consites[j2];
    frac4=(mu/(s4*pow(1+(1-s4)*recprob/s4,2))) * consites[j2];
    frac=0.1325 * frac1 + 0.0847 * frac2 + 0.1386 * frac3 + 0.6442*frac4;
    Bexp=Bexp+frac;

     //Sweeps
      prod=exp(-recprob*tau)*s_pos *subs[j2];     
     Ssum=Ssum+prod;
     prod2=exp(-recprob*tau)*s_pos *subs2[j2];     
     Ssum2=Ssum2+prod2;
     
   }
   
   B=exp(-1.0* Bexp);
  S= (1.0/splittime)*alpha*Ssum;
  S2= (1.0/splittime)*alpha2*Ssum2;
  pivector[i2]= ((2*mu)/(2*mu+1/(2*Ne*B)+S+S2)) *(1-consites[i2]/1000) + (consites[i2]/1000 * 0.0006);
  B_tot=pivector[i2]/(4*Ne*mu);
    myfile << i2 << " " << B << " " << pivector[i2] << " " << B_tot <<"\n";
  
 }

myfile.close();


// convert  pi to 200 kb windows
    int wind200= windnr/200;
  vector<double> outvector (wind200); // new vector with wanted size 
          
      int wratio= windnr/wind200; //how many in windows in one outwindow?
      int indcon; //indexconverter
      double newvalue;
      for (int p1=0; p1<wind200; p1++){ //over all out windows
	  newvalue=0.0;
	  indcon=p1*wratio;
	  for (int p2=indcon; p2<indcon+wratio; p2++){
	    newvalue=newvalue+pivector[p2];	    
	  }
	  outvector[p1]=newvalue/(wratio*(4*Ne*mu));
      }

      
 std::ofstream myfile2( outfile2.c_str() );

    for (int i=0; i<wind200; i++){
  
 myfile2 << i << " " << outvector[i] << "\n";}
 myfile2.close();



} catch (std::exception & error){ std::cout<< error.what() <<std::endl; }


